@@ -5,6 +5,7 @@ import (
 	"encoding/json"
 	"html/template"
 	"log"
+	"mime"
 	"net/http"
 	"sort"
 	"strings"
@@ -181,6 +182,8 @@ func main() {
 
 	log.Printf("Starting chromad %s on http://%s\n", version, cli.Bind)
 
+	mime.AddExtensionType(".js", "application/javascript")
+
 	router := mux.NewRouter()
 	router.Handle("/", http.HandlerFunc(index)).Methods("GET")
 	router.Handle("/api/render", http.HandlerFunc(renderHandler)).Methods("POST")
